/*
** Module   : CommonListWidget.java
** Abstract : Define all list widgets-related APIs available via a widget handle instance
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 IAS 20150209        Created initial version.
** 022 IAS 20150225          Added support for the DELIMITER attribute,
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

public interface CommonListWidget<T extends ControlSetConfig> extends CommonWidget
{

   /**
    * Implements the ENTRY() widget method.  This method returns items from
    * the set by 1-based index.
    *
    * @param    index
    *           The index of an item.
    *
    * @return   The list item indexed or <code>unknown value</code>.
    */
   @LegacyMethod(name = "ENTRY")
   public character entry(int64 index);

   /**
    * Implements the ENTRY() widget method.  This method returns items from
    * the set by 1-based index.
    *
    * @param    index
    *           The index of an item.
    *
    * @return   The list item indexed or <code>unknown value</code>.
    */
   @LegacyMethod(name = "ENTRY")
   public character entry(long index);

   /**
    * Implements the LOOKUP() widget method.  This method returns the 1-based
    * item's index in the set.
    *
    * @param    text
    *           The text of an item.
    *
    * @return   The item's index or <code>unknown value</code>.
    */
   public integer lookup(character text);

   /**
    * Implements the LOOKUP() widget method.  This method returns the 1-based
    * item's index in the set.
    *
    * @param    text
    *           The text of an item.
    *
    * @return   The item's index or <code>unknown value</code>.
    */
   public integer lookup(String text);

   /**
    * Implements the ADD-FIRST() widget method, which adds a list of labels
    * to the beginning of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character itemList);

   /**
    * Implements the ADD-FIRST() widget method, which adds a list of labels
    * to the beginning of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String itemList);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character label, BaseDataType value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String label, BaseDataType value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character label, String value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String label, String value);

   /**
    * Implements the ADD-LAST() widget method, which adds a list of labels
    * to the end of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character itemList);

   /**
    * Implements the ADD-LAST() widget method, which adds a list of labels
    * to the end of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String itemList);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character label, BaseDataType value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String label, BaseDataType value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character label, String value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String label, String value);

   /**
    * Implements the DELETE() widget method, which removes the list item or items specified.
    *
    * @param    item
    *           The item or items to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(character item);
   
   /**
    * Implements the DELETE() widget method, which removes the list item or items specified.
    *
    * @param    item
    *           The item or items to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(String item);
   
   /**
    * Implements the DELETE() widget method, which removes the list item at the given index.
    *
    * @param    idx
    *           The index of the item to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(int64 idx);
   
   /**
    * Implements the DELETE() widget method, which removes the list item at the given index.
    *
    * @param    idx
    *           The index of the item to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(long idx);
   
   /**
    * Store array of items into config.
    * 
    * @param    items
    *           Array of new items.
    */
   public void setItems(ControlSetItem[] items);

   /**
    * Sets the LIST-ITEMS writeable attribute.
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEMS", setter = true)
   public void setListItems(character list);

   /**
    * Setter for the LIST-ITEMS writeable attribute.
    *
    * @param    list
    *           The new value for the LIST-ITEMS.
    */
   @LegacyAttribute(name = "LIST-ITEMS", setter = true)
   public void setListItems(String list);

   /**
    * Common implementation list item pair attributes (LIST-ITEM-PAIRS and
    * RADIO-BUTTONS) attribute. The parameter contains string representation
    * of all labels and values separated by a delimiter (comma is the default
    * delimiter).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,...,LabelN,valueN</code>
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS", setter = true)
   public void setListItemPairs(character list);

   /**
    * Common implementation list item pair attributes (LIST-ITEM-PAIRS and
    * RADIO-BUTTONS) attribute. The parameter contains string representation
    * of all labels and values separated by a delimiter (comma is the default
    * delimiter).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,...,LabelN,valueN</code>
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS", setter = true)
   public void setListItemPairs(String list);

   /**
    * Gets the LIST-ITEMS writeable attribute.
    * 
    * @return   The current value of LIST-ITEMS attribute.
    */
   @LegacyAttribute(name = "LIST-ITEMS")
   public character getListItems();

   /**
    * Gets the LIST-ITEM-PAIRS writeable attribute.
    *
    * @return   The current value of LIST-ITEMS attribute.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS")
   public character getListItemPairs();
   
   /**
    * Gets the DELIMITER writeable attribute.
    *
    * @return   The current value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER")
   public character getDelimiter();

   /**
    * Sets the DELIMITER writeable attribute.
    *
    * @param value
    *       The new value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER", setter = true )
   public void setDelimiter(character value);
   
   /**
    * Sets the DELIMITER writeable attribute.
    *
    * @param value
    *       The new value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER", setter = true )
   public void setDelimiter(String value);

   /**
    * Tell widget that data format should be honored.
    * 
    * @param    b
    *           If <code>true</code>, the format string will be honored.
    */
   public void setHonorFormat(boolean b);
}