/*
** Module   : ScrollDirection.java
** Abstract : Scrolling direction constants and helper methods.
**
** Copyright (c) 2010-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 IAS 20150219          Some refactorings.
*/

package com.goldencode.p2j.ui.client.event;

/**
 * Scrolling direction constants.
 */
public enum ScrollDirection
{
   /** Scroll down. */
   DOWN(0, -1),
   /** Scroll down and left. */
   DOWN_LEFT(-1, -1),
   /** Scroll down and right. */
   DOWN_RIGHT(1, -1),
   /** Scroll left. */
   LEFT(-1, 0),
   /** Scroll right. */
   RIGHT(1, 0),
   /** Scroll up. */
   UP(0, 1),
   /** Scroll up and left. */
   UP_LEFT(-1, 1),
   /** Scroll up and right. */
   UP_RIGHT(1, 1);

   /** sign of the horizontal component */
   public final int x;
   /** sign of the vertical component */
   public final int y;

   /**
    * ScrollDirection constructor
    * @param x
    *       sign of x direction (-1/0/1)
    * @param y
    *       sign of y direction (-1/0/1)
    */
   private ScrollDirection(int x, int y)
   {
      this.x = x;
      this.y = y;
   }

   /** 
    * Calculate ScrollDirection for a given shift vector
    * @param dx
    *       horizontal component of the shift
    * @param dy
    *       vertical component of the shift
    * @return
    *       the shit direction or null of both components are zero.
    */
   public static ScrollDirection of(double dx, double dy)
   {
      ScrollDirection d = null;
      if (dx < 0)
      {
         if (dy < 0)
         {
            d = DOWN_LEFT;
         }
         else if (dy == 0) 
         {
            d = LEFT;
         }
         else
         {
            d = UP_LEFT;
         }
      } 
      else if( dx == 0)
      {
         if (dy < 0)
         {
            d = DOWN;
         }
         else if (dy == 0) 
         {
            d = null;
         }
         else
         {
            d = UP;
         }
      }
      else 
      {
         if (dy < 0)
         {
            d = DOWN_RIGHT;
         }
         else if (dy == 0) 
         {
            d = RIGHT;
         }
         else
         {
            d = UP_RIGHT;
         }
         
      }
      return d;
   }
   /**
    * Check is specified direction contains {@link #UP} component.
    * 
    * @param   direction
    *          Direction to check.
    *          
    * @return  <code>true</code> if direction contains {@link #UP} component.
    */
   public static boolean up(ScrollDirection direction)
   {
      return direction != null && direction.y > 0;
   }

   /**
    * Check is specified direction contains {@link #DOWN} component.
    * 
    * @param   direction
    *          Direction to check.
    *          
    * @return  <code>true</code> if direction contains {@link #DOWN} component.
    */
   public static boolean down(ScrollDirection direction)
   {
      return direction != null && direction.y < 0;
   }

   /**
    * Check is specified direction contains {@link #RIGHT} component.
    * 
    * @param   direction
    *          Direction to check.
    *          
    * @return  <code>true</code> if direction contains {@link #RIGHT} component.
    */
   public static boolean right(ScrollDirection direction)
   {
      return direction != null && direction.x > 0;
   }

   /**
    * Check is specified direction contains {@link #LEFT} component.
    * 
    * @param   direction
    *          Direction to check.
    * 
    * @return  <code>true</code> if direction contains {@link #LEFT} component.
    */
   public static boolean left(ScrollDirection direction)
   {
      return direction != null && direction.x < 0;
   }
}
